package com.zd.laboratory.mapper;

import com.zd.laboratory.domain.LabBuilding;
import com.zd.laboratory.domain.vo.LabBuildingInfoVO;
import com.zd.laboratory.domain.vo.LabBuildingVO;
import com.zd.laboratory.domain.vo.LabSubjectLayoutVo;

import java.util.List;

/**
 * 楼栋Mapper接口
 *
 * @author zd
 * @date 2021-09-01
 */
public interface LabBuildingMapper {
    /**
     * 查询楼栋
     *
     * @param id 楼栋主键
     * @return 楼栋
     */
    public LabBuilding selectLabBuildingById(Long id);

    /**
     * 查询楼栋列表
     *
     * @param labBuilding 楼栋
     * @return 楼栋集合
     */
    public List<LabBuilding> selectLabBuildingList(LabBuilding labBuilding);

    /**
     * 新增楼栋
     *
     * @param labBuilding 楼栋
     * @return 结果
     */
    public int insertLabBuilding(LabBuilding labBuilding);

    /**
     * 修改楼栋
     *
     * @param labBuilding 楼栋
     * @return 结果
     */
    public int updateLabBuilding(LabBuilding labBuilding);

    /**
     * 删除楼栋
     *
     * @param id 楼栋主键
     * @return 结果
     */
    public int deleteLabBuildingById(Long id);

    /**
     * 批量删除楼栋
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteLabBuildingByIds(Long[] ids);


    /**
     * 查询楼栋列表(不分页)
     *
     * @param labBuilding 楼栋
     * @return 楼栋集合
     */
    public List<LabBuilding> getBuildLlist(LabBuilding labBuilding);


    /**
     * 查询楼栋列表
     *
     * @return 楼栋集合
     */
    public List<LabBuildingVO> labLayout(LabBuilding labBuilding);

    List<LabBuildingInfoVO> queryLabInfoLayout(LabBuildingVO labBuilding);

}
